@@ -4,14 +4,14 @@ use 5.008_001;
 use strict;
 use warnings;
 
-our $VERSION = '0.1999_06';
+our $VERSION = '0.2008_01';
 
-use Carp        ();
-use File::Spec  ();
-use Exporter    ();
+use Carp              ();
+use File::Spec        ();
+use Exporter          ();
+use Data::MessagePack ();
 
 use Text::Xslate::Util qw(
-    $DEBUG
     mark_raw unmark_raw
     html_escape escaped_string
     uri_escape html_builder
@@ -29,15 +29,16 @@ our @EXPORT_OK = qw(
 );
 
 # load backend (XS or PP)
-if(!__PACKAGE__->can('render')) { # The backend is already loaded
-    if($DEBUG !~ /\b pp \b/xms) {
+if(!exists $INC{'Text/Xslate/PP.pm'}) {
+    my $pp = ($Text::Xslate::Util::DEBUG =~ /\b pp \b/xms or $ENV{PERL_ONLY});
+    if(!$pp) {
         eval {
             require XSLoader;
             XSLoader::load(__PACKAGE__, $VERSION);
         };
-        die $@ if $@ && $DEBUG =~ /\b xs \b/xms; # force XS
+        die $@ if $@ && $Text::Xslate::Util::DEBUG =~ /\b xs \b/xms; # force XS
     }
-    if(!__PACKAGE__->can('render')) { # failed to load XS, or force PP
+    if(!__PACKAGE__->can('render')) {
         require 'Text/Xslate/PP.pm';
     }
 }
@@ -45,8 +46,6 @@ if(!__PACKAGE__->can('render')) { # The backend is already loaded
 package Text::Xslate::Engine;
 
 use Text::Xslate::Util qw(
-    $NUMBER $STRING $DEBUG
-    literal_to_value
     import_from
     make_error
 );
@@ -54,7 +53,7 @@ use Text::Xslate::Util qw(
 BEGIN {
     our @ISA = qw(Exporter);
 
-    my $dump_load = scalar($DEBUG =~ /\b dump=load \b/xms);
+    my $dump_load = scalar($Text::Xslate::Util::DEBUG =~ /\b dump=load \b/xms);
     *_DUMP_LOAD = sub(){ $dump_load };
 
     *_ST_MTIME = sub() { 9 }; # see perldoc -f stat
@@ -186,12 +185,6 @@ sub _merge_hash {
 }
 
 
-sub flush_memory_cache {
-    my($self) = @_;
-    %{$self->{template}} = ();
-    return;
-}
-
 sub load_string { # for <string>
     my($self, $string) = @_;
     if(not defined $string) {
@@ -222,6 +215,8 @@ sub find_file {
         if(ref $p eq 'HASH') { # virtual path
             defined(my $content = $p->{$file}) or next;
             $fullpath   = \$content;
+            # TODO: we should provide a way to specify this mtime
+            #       (like as Template::Provider?)
             $orig_mtime = $^T;
             $path_id    = 'HASH';
         }
@@ -259,7 +254,9 @@ sub find_file {
 
 
 sub load_file {
-    my($self, $file, $mtime) = @_;
+    my($self, $file, $mtime, $from_include) = @_;
+
+    local $self->{from_include} = $from_include;
 
     print STDOUT "load_file($file)\n" if _DUMP_LOAD;
 
@@ -319,9 +316,8 @@ sub _load_source {
                 or Carp::carp("Xslate: Cannot make directory $cachepath (ignored): $@");
         }
 
-        # use input_layer for caches
-        if(open my($out), '>' . $self->{input_layer}, $cachepath) {
-            $self->_save_compiled($out, $asm, $fullpath);
+        if(open my($out), '>:raw', $cachepath) {
+            $self->_save_compiled($out, $asm, $fullpath, utf8::is_utf8($source));
 
             if(!close $out) {
                  Carp::carp("Xslate: Cannot close $cachepath (ignored): $!");
@@ -364,57 +360,49 @@ sub _load_compiled {
     }
 
     my $cachepath = $fi->{cachepath};
-    open my($in), '<' . $self->{input_layer}, $cachepath
+    open my($in), '<:raw', $cachepath
         or $self->_error("LoadError: Cannot open $cachepath for reading: $!");
 
     if(scalar(<$in>) ne $self->_magic_token($fi->{fullpath})) {
         return undef;
     }
 
-    # parse assembly
+    my $data;
+    {
+        local $/;
+        $data = <$in>;
+        close $in;
+    }
+    my $unpacker = Data::MessagePack::Unpacker->new();
+    my $offset  = $unpacker->execute($data);
+    my $is_utf8 = $unpacker->data();
+    $unpacker->reset();
+
     my @asm;
-    while(defined(my $s = <$in>)) {
-        next if $s =~ m{\A [ \t]* (?: \# | // )}xms; # comments
-        chomp $s;
+    while($offset < length($data)) {
+        $offset = $unpacker->execute($data, $offset);
+        my $c = $unpacker->data();
+        $unpacker->reset();
 
-        # See ::Compiler::as_assembly()
-        # "$opname $arg #$line:$file *$symbol // $comment"
-
-        my($name, $value, $line, $file, $symbol) = $s =~ m{
-            \A
-                [ \t]*
-                ($IDENT)                        # an opname
-
-                # the following components are optional
-                (?: [ \t]+ ($STRING|[+-]?$NUMBER) )? # operand
-                (?: [ \t]+ \#($NUMBER)          # line number
-                    (?: [:] ($STRING))?         # file name
-                )?
-                (?: [ \t]+ \*($STRING) )?       # symbol name
-                (?: [ \t]* // [^\n]*)?          # comments (anything)
-            \z
-        }xmsog or $self->_error("LoadError: Cannot parse assembly (line $.): $s");
-
-        $value = literal_to_value($value);
-
-        # checks the modified of dependencies
-        if($name eq 'depend') {
-            my $dep_mtime = (stat $value)[_ST_MTIME];
+        utf8::decode($c->[1]) if $is_utf8 && defined $c->[1];
+        # my($name, $arg, $line, $file, $symbol) = @{$c};
+        if($c->[0] eq 'depend') {
+            my $arg = $c->[1];
+            my $dep_mtime = (stat $arg)[_ST_MTIME];
             if(!defined $dep_mtime) {
                 $dep_mtime = 9**9**9; # force reload
-                Carp::carp("Xslate: Failed to stat $value (ignored): $!");
+                Carp::carp("Xslate: Failed to stat $arg (ignored): $!");
             }
             if($dep_mtime > $threshold){
                 printf "  _load_compiled: %s(%s) is newer than %s(%s)\n",
-                    $value,     scalar localtime($dep_mtime),
+                    $arg,       scalar localtime($dep_mtime),
                     $cachepath, scalar localtime($threshold)
                         if _DUMP_LOAD;
 
                 return undef;
             }
         }
-
-        push @asm, [ $name, $value, $line, $file, $symbol ];
+        push @asm, $c;
     }
 
     if(_DUMP_LOAD) {
@@ -426,8 +414,12 @@ sub _load_compiled {
 }
 
 sub _save_compiled {
-    my($self, $out, $asm, $fullpath) = @_;
-    print $out $self->_magic_token($fullpath), $self->_compiler->as_assembly($asm);
+    my($self, $out, $asm, $fullpath, $is_utf8) = @_;
+    print $out $self->_magic_token($fullpath);
+    print $out Data::MessagePack->pack($is_utf8 ? 1 : 0);
+    foreach my $c(@{$asm}) {
+        print $out Data::MessagePack->pack($c);
+    }
     return;
 }
 
@@ -488,7 +480,7 @@ sub _compiler {
 
 sub compile {
     my $self = shift;
-    return $self->_compiler->compile(@_);
+    return $self->_compiler->compile(@_, from_include => $self->{from_include});
 }
 
 sub _error {
@@ -509,12 +501,11 @@ Text::Xslate - Scalable template engine for Perl5
 
 =head1 VERSION
 
-This document describes Text::Xslate version 0.1999_06.
+This document describes Text::Xslate version 0.2008_01.
 
 =head1 SYNOPSIS
 
     use Text::Xslate;
-    use FindBin qw($Bin);
 
     my $tx = Text::Xslate->new(
         # the fillowing options are optional.
@@ -572,12 +563,12 @@ are available.
 
     $ perl -Mblib benchmark/x-rich-env.pl
     Perl/5.10.1 i686-linux
-    Text::Xslate/0.1055
-    Text::MicroTemplate/0.13
+    Text::Xslate/0.2002
+    Text::MicroTemplate/0.18
     Text::MicroTemplate::Extended/0.11
     Template/2.22
     Text::ClearSilver/0.10.5.4
-    HTML::Template::Pro/0.9502
+    HTML::Template::Pro/0.9503
     1..4
     ok 1 - TT: Template-Toolkit
     ok 2 - MT: Text::MicroTemplate
@@ -585,11 +576,11 @@ are available.
     ok 4 - HTP: HTML::Template::Pro
     Benchmarks with 'include' (datasize=100)
               Rate     TT     MT    TCS    HTP Xslate
-    TT       131/s     --   -71%   -94%   -95%   -99%
-    MT       444/s   240%     --   -80%   -84%   -97%
-    TCS     2181/s  1570%   391%     --   -20%   -88%
-    HTP     2739/s  1997%   516%    26%     --   -85%
-    Xslate 17772/s 13505%  3899%   715%   549%     --
+    TT       129/s     --   -84%   -94%   -95%   -99%
+    MT       807/s   527%     --   -63%   -71%   -96%
+    TCS     2162/s  1580%   168%     --   -23%   -89%
+    HTP     2814/s  2087%   249%    30%     --   -85%
+    Xslate 19321/s 14912%  2295%   794%   587%     --
 
 According to this result, Xslate is 100+ times faster than Template-Toolkit.
 Text::MicroTemplate is a very fast template engine written in pure Perl, but
@@ -601,8 +592,9 @@ There are benchmark scripts in the F<benchmark/> directory.
 
 =head3 Auto escaping to HTML meta characters
 
-All the template expressions the engine interpolates into templates are
-html-escaped automatically, so the output has no possibility to XSS by default.
+All the HTML meta characters in template expressions the engine interpolates
+into template texts are automatically escaped, so the output has no
+possibility to XSS by default.
 
 =head3 Template cascading
 
@@ -632,7 +624,7 @@ Possible options are:
 =item C<< path => \@path // ['.'] >>
 
 Specifies the include paths, which may be directory names or virtual paths,
-i.e. HASH references which contain C<< $file_name => $content >> mapping.
+i.e. HASH references which contain C<< $file_name => $content >> pairs.
 
 =item C<< cache => $level // 1 >>
 
@@ -644,7 +636,7 @@ checks the freshness of the original templates every time.
 If I<$level> E<gt>= 2, caches will be created but the freshness
 will not be checked.
 
-I<$level> == 0 creates no caches. It's provided for testing.
+I<$level> == 0 uses no caches. It's provided for testing.
 
 =item C<< cache_dir => $dir // "$ENV{HOME}/.xslate_cache" >>
 
@@ -658,7 +650,7 @@ You B<should> specify this option on productions.
 Specifies a function map which contains name-coderef pairs.
 A function C<f> may be called as C<f($arg)> or C<$arg | f> in templates.
 
-There are a few builtin filters, but they are not overridable.
+There are builtin filters which are not overridable.
 
 =item C<< module => [$module => ?\@import_args, ...] >>
 
@@ -843,7 +835,7 @@ This function is available in templates as the C<unmark_raw> filter.
 
 =head3 C<< html_escape($str :Str) :RawStr >>
 
-Escapes html meta characters in I<$str>, and returns it as a raw string (see above).
+Escapes HTML meta characters in I<$str>, and returns it as a raw string (see above).
 If I<$str> is already a raw string, it returns I<$str> as is.
 
 By default, this function will be automatically applied to all the template
