@@ -3,14 +3,13 @@ package Text::Xslate::PP;
 use 5.008_001;
 use strict;
 
-our $VERSION = '0.1999_06';
+our $VERSION = '0.2008_01';
 
 BEGIN{
     $ENV{XSLATE} = ($ENV{XSLATE} || '') . '[pp]';
 }
 use Text::Xslate::Util qw(
     $DEBUG
-    $NUMBER $STRING
     p
 );
 
@@ -26,7 +25,7 @@ use constant _DUMP_LOAD => scalar($DEBUG =~ /\b dump=load \b/xms);
 use Text::Xslate::PP::Const qw(:all);
 use Text::Xslate::PP::State;
 use Text::Xslate::PP::Type::Raw;
-use Text::Xslate;
+use Text::Xslate ();
 
 use Carp ();
 
@@ -79,7 +78,7 @@ sub options {
 sub render_string {
     my($self, $string, $vars) = @_;
     $self->load_string($string);
-    return $self->render(undef, $vars);
+    return $self->render('<string>', $vars);
 }
 
 sub render {
@@ -87,20 +86,23 @@ sub render {
 
     Carp::croak("Usage: Text::Xslate::render(self, name, vars)")
         if !( @_ == 2 or @_ == 3 );
+    unless ( ref $self ) {
+        Carp::croak( "Invalid xslate instance" );
+    }
 
     if(!defined $vars) {
         $vars = {};
     }
 
     if ( !defined $name ) {
-        $name = '<string>';
+        Carp::croak("Xslate: Template name is not given");
     }
 
     unless ( ref $vars eq 'HASH' ) {
         Carp::croak( sprintf("Xslate: Template variables must be a HASH reference, not %s", $vars ) );
     }
 
-    my $st = tx_load_template( $self, $name );
+    my $st = tx_load_template( $self, $name, 0 );
 
     local $_orig_die_handler  = $SIG{__DIE__};
     local $_orig_warn_handler = $SIG{__WARN__};
@@ -435,7 +437,7 @@ sub tx_push_frame {
 
 
 sub tx_load_template {
-    my ( $self, $name ) = @_;
+    my ( $self, $name, $from_include ) = @_;
 
     unless ( ref $self ) {
         Carp::croak( "Invalid xslate instance" );
@@ -458,7 +460,7 @@ sub tx_load_template {
     }
 
     if ( not exists $ttable->{ $name } ) {
-        $self->load_file( $name );
+        $self->load_file( $name, undef, $from_include );
         $retried++;
         goto RETRY;
     }
@@ -481,7 +483,7 @@ sub tx_load_template {
         return $self->{ tmpl_st }->{ $name };
     }
     else{
-        $self->load_file( $name, $cache_mtime );
+        $self->load_file( $name, $cache_mtime, $from_include );
         $retried++;
         goto RETRY;
     }
@@ -626,7 +628,7 @@ Text::Xslate::PP - Yet another Text::Xslate runtime in pure Perl
 
 =head1 VERSION
 
-This document describes Text::Xslate::PP version 0.1999_06.
+This document describes Text::Xslate::PP version 0.2008_01.
 
 =head1 DESCRIPTION
 
